SystemOrganization addCategory: #DAVServerDirectory!
SystemOrganization addCategory: #'DAVServerDirectory-SVNObjects'!

ProtocolClient subclass: #DAVClient
	instanceVariableNames: 'lastHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAVServerDirectory'!

!DAVClient commentStamp: 'tak 7/10/2006 16:48' prior: 0!
This class implements HTTP/1.1 + WebDAV protocol
http://www.ietf.org/rfc/rfc2518.txt

self openUrl: 'http://google.com' asUrl while: [:client | client get]

Structure:
	lastHeader		Dictionary - response header which will be used for error handling like 404 or 301.
!

!DAVClient class methodsFor: 'accessing' stamp: 'tak 7/7/2006 12:44'!
defaultPortNumber
	^ 80! !

!DAVClient class methodsFor: 'instance creation' stamp: 'tak 7/7/2006 15:15'!
openOnHostNamed: hostName port: portNumber 
	| client |
	client := super
				openOnHostNamed: hostName
				port: (portNumber
						ifNil: [self defaultPortNumber]).
	client hostName: hostName.
	^ client! !

!DAVClient class methodsFor: 'instance creation' stamp: 'tak 7/8/2006 15:46'!
openUrl: aUrl 
	^ self new openUrl: aUrl! !

!DAVClient class methodsFor: 'public protocol' stamp: 'tak 7/8/2006 16:00'!
openUrl: aUrl while: aBlock 
	| client |
	client := self openUrl: aUrl.
	[^ aBlock value: client]
		ensure: [client close]! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:42'!
absPath
	^ self url fullPath! !

!DAVClient methodsFor: 'public protocol' stamp: 'yo 9/9/2009 01:22'!
allLastResponseCodeFrom: aString
	"'HTTP/1.1 405 Method Not Allowed' copyFrom: 10 to: 12"
	| c xml |
	self multistatus ifTrue: [
		c := OrderedCollection new.
		xml := (XMLDOMParser parseDocumentFrom: aString readStream useNamespaces: true) .
			c := OrderedCollection new.
			xml withAllElementsDo: [:e |
				(e name asLowercase endsWith: 'd:status') ifTrue: [c add: e contents first asString]].
			^ c collect: [:s | s copyFrom: 10 to: 12]]
	ifFalse: [
		^ Array with: (self lastResponse copyFrom: 10 to: 12)].
! !

!DAVClient methodsFor: 'private' stamp: 'yo 8/6/2009 18:15'!
askName

	self user: (FillInTheBlankMorph
					request: 'User account name?'
					initialAnswer: (self user ifNil: [''])
					onCancelReturn: nil).
	Utilities authorName: self user.
! !

!DAVClient methodsFor: 'private' stamp: 'yo 8/6/2009 18:16'!
askNamePassword

	self askName.
	self askPassword.
! !

!DAVClient methodsFor: 'private' stamp: 'yo 8/6/2009 19:44'!
askNamePasswordIfNecessary
	lastHeader ifNil: [^ true].
	(self user isNil or: [self password isNil]) ifFalse: [^ true].
	self askNamePassword.
! !

!DAVClient methodsFor: 'private' stamp: 'yo 8/6/2009 18:16'!
askPassword

	self password: (FillInTheBlank  requestPassword: 'Password?').
	^ true! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/7/2006 12:53'!
checkResponse
! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 15:44'!
delete
	^ self
		method: 'DELETE'
		header: Dictionary new
		body: ''! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 15:53'!
get
	^ self
		method: 'GET'
		header: Dictionary new
		body: ''! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:34'!
hostName
	^ self url
		ifNotNilDo: [:url | url authority]! !

!DAVClient methodsFor: 'as yet unclassified' stamp: 'yo 9/9/2009 01:23'!
initialize

	super initialize.
	multistatus := false.
! !

!DAVClient methodsFor: 'private protocol' stamp: 'yo 8/6/2009 15:50'!
isRetryNeeded
	self lastResponse isEmpty
		ifTrue: [self error: 'no content'.
			^ true].
	((lastHeader includesKey: 'connection')
			and: [(lastHeader at: 'connection')
					= 'close'])
		ifTrue: [self close.
			self ensureConnection].
	self lastResponseCode first = $2
		ifTrue: [^ false].
	self lastResponseCode = '401'
		ifTrue: [self askNamePasswordIfNecessary.
			self password isEmptyOrNil ifTrue: [^false].
			^ (HTTPLoginFailedException protocolInstance: self)
				signal: self lastResponse].
	(self lastResponseCode first = $3
			and: [lastHeader includesKey: 'location'])
		ifTrue: [self openUrl: (lastHeader at: 'location') asUrl.
			^ true].
	self lastResponseCode first = $5
		ifTrue: [self inform: ('An Internal Server Error Occured ({1})' translated format: {self lastResponseCode printString}).
			^ true].
	(ProtocolClientError protocolInstance: self)
		signal: self lastResponse.
	^ false! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 18:23'!
lastHeader
	^ lastHeader! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/7/2006 14:28'!
lastResponseCode
	"'HTTP/1.1 405 Method Not Allowed' copyFrom: 10 to: 12"
	^ self lastResponse
		ifNotNil: [self lastResponse copyFrom: 10 to: 12]! !

!DAVClient methodsFor: 'private' stamp: 'yo 9/9/2009 01:23'!
lastResponse: aString
	lastResponse _ aString.
	multistatus := (lastResponse asLowercase endsWith: 'multistatus') or: [lastResponse asLowercase endsWith:'multi-status'].
! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/7/2006 12:53'!
login
! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 16:10'!
logReportFrom: fromNumber to: toNumber depth: depthNumber 
	| headers body |
	headers := Dictionary new.
	depthNumber
		ifNotNil: [headers at: 'Depth' put: depthNumber asString].
	body := '<S:log-report xmlns:S="svn:">
<S:start-revision>{1}</S:start-revision>
<S:end-revision>{2}</S:end-revision>
<S:path></S:path>
</S:log-report>' format: {fromNumber asString. toNumber asString}.
	^ self
		method: 'REPORT'
		header: headers
		body: body! !

!DAVClient methodsFor: 'private protocol' stamp: 'yo 9/9/2009 00:36'!
method: method header: aDictionary body: aStringOrByteArray 
	"(self openUrl: 'http://localhost:8080/svn/' asUrl) method: 'GET' header: Dictionary new body: ''"
	| body |
	self askNamePasswordIfNecessary.
	[self
		sendRequest: method
		header: aDictionary
		body: aStringOrByteArray.
	body := self receiveResponse.
	self isRetryNeeded] whileTrue.
	^ body! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 15:53'!
mkcol
	^ self
		method: 'MKCOL'
		header: Dictionary new
		body: ''! !

!DAVClient methodsFor: 'accessing' stamp: 'yo 9/9/2009 01:23'!
multistatus

	^ multistatus
! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/7/2006 15:54'!
nextChunkedBody
	| remain writer chunkSize size |
	remain := true.
	writer := WriteStream on: (String new: 1000).
	[remain]
		whileTrue: [chunkSize := self stream nextLine.
			size := ('16r' , chunkSize asUppercase) asNumber.
			size = 0
				ifTrue: ["last-chunk"
					remain := false]
				ifFalse: ["chunk-data"
					writer
						nextPutAll: (self stream next: size).
					"skip CRLF"
					self stream next: 2]].
	"skip CRLF"
	self stream next: 2.
	^ writer contents! !

!DAVClient methodsFor: 'public protocol' stamp: 'yo 9/16/2009 17:00'!
openUrl: aUrl 
	"self new openUrl: 'http://localhost/some/path.html' asUrl"
	| newPort addr |
	newPort := aUrl port
				ifNil: [self defaultPortNumber].
	(aUrl authority = self hostName
			and: [newPort = self port])
		ifFalse: [self close].
	self url: aUrl.
	addr := [NetNameResolver addressForName: aUrl authority timeout: 20] on: Error do: [:ex | nil].
	addr ifNil: [NetworkError signal].
	self
		openOnHost: addr
		port: newPort! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/7/2006 13:54'!
parseHttpHeader: string
	"Copy from HttpRequest >> parseHttpHeader: in KomHttpServer package by gk"
	| dict key value start end more colonPos fieldDelims lastKey keyValueDelim crlf |
	dict _ Dictionary new.
	crlf _ String crlf.
	string isEmptyOrNil ifTrue: [^dict].
	fieldDelims _ crlf asCharacterSet.
	keyValueDelim _ $:.
	more _ true.
	start _ end _ 1.
	lastKey _ ''.
	[end _ string indexOfAnyOf: fieldDelims startingAt: start.
	end == 0
		ifTrue: [end _ string size. more _ false]
		ifFalse: [end _ end - 1].
	(end >= start and: [start < string size])
		ifTrue: [
			(string at: start) isSeparator
				ifTrue:
					[dict
						at: lastKey
						modify: [:item | item, (string copyFrom: start to: end)]]
				ifFalse:
					[colonPos _ string indexOf: keyValueDelim startingAt: start.
					(colonPos > end or: [colonPos == 0])
						ifTrue:
							[key _ (string copyFrom: start to: end) translateToLowercase.
							value _ '']
						ifFalse:
							[key _ (string copyFrom: start to: colonPos-1) translateToLowercase.
							value _ (string copyFrom: colonPos+1 to: end) withBlanksTrimmed]].
			key isEmpty
				ifFalse: [dict at: key put: value.
					lastKey _ key.
					key _ ''].
			start _ string skipDelimiters: crlf startingAt: end+1].
	more] whileTrue.

	^dict! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:40'!
password
	^ self url password! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:40'!
password: aString
	^ self url password: aString! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/9/2006 00:29'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPut: $[.
	aStream
		nextPutAll: (self isConnected
				ifTrue: [self stream socket statusString]
				ifFalse: ['disconnected']).
	aStream nextPut: $]! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 16:08'!
propFind: propertyKeys depth: depthNumber label: labelNumber 
	| reqStream contents headers |
	reqStream := '' writeStream.
	reqStream nextPutAll: '<propfind xmlns="DAV:"><prop>'.
	propertyKeys
		do: [:each | reqStream nextPutAll: '<' , each , '/>'].
	reqStream nextPutAll: '</prop></propfind>'.
	contents := reqStream contents.
	headers := Dictionary new.
	depthNumber
		ifNotNil: [headers at: 'Depth' put: depthNumber asString].
	labelNumber
		ifNotNil: [headers at: 'Label' put: labelNumber asString].
	^ self
		method: 'PROPFIND'
		header: headers
		body: contents! !

!DAVClient methodsFor: 'public protocol' stamp: 'tak 7/8/2006 16:27'!
put: aStringOrByteArray
	^ self
		method: 'PUT'
		header: Dictionary new
		body: aStringOrByteArray! !

!DAVClient methodsFor: 'public protocol' stamp: 'yo 8/3/2009 19:37'!
quit

	self close.
! !

!DAVClient methodsFor: 'private' stamp: 'tak 7/9/2006 00:08'!
realm
	"so naive..."
	^ 'http://' , self hostName , ':' , self port asString! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/10/2006 22:21'!
receiveResponse
	self fetchNextResponse.
	lastHeader := self
				parseHttpHeader: (stream upToAll: String crlf , String crlf).
	(lastHeader includesKey: 'content-length')
		ifTrue: [^ stream next: (lastHeader at: 'content-length') asNumber].
	(lastHeader includesKey: 'transfer-encoding')
		ifTrue: [^ self nextChunkedBody].
	^ ''! !

!DAVClient methodsFor: 'private protocol' stamp: 'tak 7/10/2006 16:49'!
sendRequest: method header: aDictionary body: aStringOrByteArray 
	self sendCommand: method , ' ' , self absPath , ' HTTP/1.1'.
	aDictionary at: 'Content-Length' put: aStringOrByteArray size asString.
	aDictionary at: 'Host' put: self hostName.
	self user
		ifNotNil: [aDictionary at: 'Authorization' put: 'Basic ' , (Base64MimeConverter mimeEncode: (self user , ':' , self password) readStream) contents].
	aDictionary
		keysAndValuesDo: [:key :value | stream nextPutAll: key , ': ' , value , String crlf].
	stream nextPutAll: String crlf;
		 flush.
	stream nextPutAll: aStringOrByteArray;
		 flush.
! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:34'!
url
	^self connectionInfo at: #url ifAbsent: [nil]! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:32'!
url: aUrl
	^self connectionInfo at: #url put: aUrl! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:39'!
user
	^ self url username! !

!DAVClient methodsFor: 'accessing' stamp: 'tak 7/8/2006 15:39'!
user: aString
	^ self url username: aString! !

Object subclass: #SVNRevision
	instanceVariableNames: 'name location creatorDisplayName versionName date comment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAVServerDirectory-SVNObjects'!

!SVNRevision class methodsFor: 'as yet unclassified' stamp: 'tak 6/10/2006 23:49'!
newFromXMLElement: anXMLElement forFileNamed: n

	| vn cr com d inst |
	cr := ''.
	anXMLElement elements do: [:elem |
		elem name = 'D:version-name' ifTrue: [vn _ elem contents first string asNumber].
		elem name = 'D:creator-displayname' ifTrue: [cr _ elem contents first string].
		elem name = 'D:comment' ifTrue: [com _ elem contents first string].
		elem name = 'S:date' ifTrue: [d _ elem contents first string].
	].
	(vn notNil and: [com notNil and: [d notNil]]) ifTrue: [
		inst _ self new.
		inst name: n versionName: vn creatorDisplayName: cr comment: com date: d.
		^ inst.
	].
	^ nil.
! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 16:52'!
comment

	^ comment.
! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 17:00'!
creatorDisplayName

	^ creatorDisplayName.
! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 16:54'!
date

	^ date.
! !

!SVNRevision methodsFor: 'accessing' stamp: 'tak 7/12/2006 15:46'!
location
	^ location! !

!SVNRevision methodsFor: 'accessing' stamp: 'tak 7/12/2006 15:46'!
location: aString
	^ location := aString! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 16:51'!
name

	^ name.
! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 17:02'!
name: n versionName: vn creatorDisplayName: cr comment: com date: d

	name _ n.
	versionName _ vn.
	creatorDisplayName _ cr.
	comment _ com.
	d isString ifTrue: [
		date _ DateAndTime fromString: d.
	] ifFalse: [
		date _ d.
	].
! !

!SVNRevision methodsFor: 'as yet unclassified' stamp: 'yo 5/16/2006 16:52'!
versionName

	^ versionName.
! !

HTTPServerDirectory subclass: #DAVServerDirectory
	instanceVariableNames: ''
	classVariableNames: 'EntryCache'
	poolDictionaries: ''
	category: 'DAVServerDirectory'!

DAVServerDirectory subclass: #DAVMultiUserServerDirectory
	instanceVariableNames: 'setupSelector useDefaultAccount loggedIn origDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAVServerDirectory'!

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 21:29'!
askName

	| ret |
	ret := FillInTheBlankMorph
					request: (useDefaultAccount ifTrue: [
						'User account name? (Cancel to stay with default name)'] ifFalse: ['User account name?'])
					initialAnswer: (user ifNil: [''])
					onCancelReturn: nil.
	(useDefaultAccount and: [ret isNil or: [ret = user]]) ifTrue: [^ false].
	self user: ret.
	passwordHolder := nil.
	Utilities authorName: user.
	^ true.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 20:18'!
askNamePassword

	self askName ifTrue: [self askPassword]! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 19:49'!
askNamePasswordIfNecessary

	(self user isNil or: [self password isNil]) ifFalse: [^ true].
	self askNamePassword.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 19:49'!
askPassword

	self password: (FillInTheBlank  requestPassword: 'Password?').
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 19:49'!
asServerFileNamed: aName

	| rFile |
	rFile _ self as: DAVServerFile.
	(aName includes: self pathNameDelimiter)
		ifTrue: [rFile fullPath: aName]
			"sets server, directory(path), fileName.  If relative, merge with self."
		ifFalse: [rFile fileName: aName].	"JUST a single NAME, already have the rest"
			"Mac files that include / in name, must encode it as %2F "
	^rFile
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/11/2009 19:10'!
createPersonalDirectory: aString

	aString ifNil: [^ self].
	directory := origDirectory, '/', self pathToUserAccount.
	altURL := server,  directory.
	self createDirectory: aString.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 20:58'!
initialize

	super initialize.
	self setupSelector: #setupSharedDirectory:.
	useDefaultAccount := false.
	loggedIn := false.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/8/2009 19:52'!
origDirectory: aString

	origDirectory := aString
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/8/2009 18:15'!
password

	user ifNil: [passwordHolder := nil. (HTTPLoginFailedException protocolInstance: DAVClient new) signal. ^ self].
	passwordHolder ifNil: [passwordHolder _ Password new].
	passwordHolder isString ifTrue: [^ passwordHolder].
	^ passwordHolder passwordFor: self	"may ask the user"! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/11/2009 19:10'!
pathToUserAccount

	^  'everyone/accounts/'! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 21:05'!
setUserAndPassIn: aUrl

	aUrl username: self user.
	aUrl password: self password.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/11/2009 19:10'!
setupPersonalDirectory: aString

	aString ifNil: [^ self].
	directory := origDirectory, '/', self pathToUserAccount, aString.
	altURL := server,  directory.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 15:08'!
setupSelector: aSymbol

	setupSelector := aSymbol
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 15:32'!
setupSharedDirectory: aString
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 19:51'!
useDefaultAccount: aBoolean

	useDefaultAccount := aBoolean.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 9/8/2009 10:57'!
user

	| orig |
	orig := user.
	(useDefaultAccount not and: [user isNil]) ifTrue: [self askName ifFalse: [self user: orig]].
	^ user.
! !

!DAVMultiUserServerDirectory methodsFor: 'as yet unclassified' stamp: 'yo 8/6/2009 20:48'!
user: aString

	super user: aString.
	self perform: setupSelector with: aString.
! !

!DAVServerDirectory class methodsFor: 'accessing' stamp: 'tak 4/18/2006 11:25'!
entryCache
	^ EntryCache
		ifNil: [EntryCache := OrderedCollection new]! !

!DAVServerDirectory class methodsFor: 'accessing' stamp: 'tak 4/18/2006 11:25'!
entryCache: aCollection
	^ EntryCache := aCollection! !

!DAVServerDirectory class methodsFor: 'accessing' stamp: 'tak 4/18/2006 12:58'!
extensionDelimiter
	^ FileDirectory extensionDelimiter! !

!DAVServerDirectory class methodsFor: 'instance creation' stamp: 'tak 4/17/2006 14:18'!
newFrom: aSimilarObject
	^super newFrom: aSimilarObject! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'cds 2/8/2005 15:23'!
asServerFileNamed: aName

	| rFile |
	rFile _ self as: DAVServerFile.
	(aName includes: self pathNameDelimiter)
		ifTrue: [rFile fullPath: aName]
			"sets server, directory(path), fileName.  If relative, merge with self."
		ifFalse: [rFile fileName: aName].	"JUST a single NAME, already have the rest"
			"Mac files that include / in name, must encode it as %2F "
	^rFile
! !

!DAVServerDirectory methodsFor: 'accessing' stamp: 'yo 8/6/2009 20:14'!
asUrl
	| newUrl |
	newUrl _ ('http://' , self server , '/' , self directory, '/') asUrl.
	self setUserAndPassIn: newUrl.
	^ newUrl.
! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 20:12'!
createDirectory: localName 
	| url |
	url := ('http://', self server , '/' , self directory , '/' , localName) asUrl.
	self setUserAndPassIn: url.
	DAVClient openUrl: url while: [ :c | c mkcol]! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 20:12'!
deleteDirectory: fullName 
	| url |
	url := (self server , '/' , self directory , '/' , fullName , '/') asUrl.
	self setUserAndPassIn: url.
	DAVClient openUrl: url while: [ :c | c delete]! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 20:12'!
deleteFileNamed: fullName 
	| url |
	url := (self server , '/' , self directory , '/' , fullName) asUrl.
	self setUserAndPassIn: url.
	DAVClient openUrl: url while: [ :c | c delete]! !

!DAVServerDirectory methodsFor: 'accessing' stamp: 'tak 4/18/2006 15:20'!
directory: aString 
	| dir |
	dir _ aString.
	(dir isEmpty not
			and: [dir last = $/])
		ifTrue: [dir _ dir allButLast].
	(dir isEmpty not
			and: [dir first = $/])
		ifTrue: [dir _ dir allButFirst].
	super directory: dir! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 21:30'!
entries
	"EntryCache -- a collection of {time. url. directory index}"
	| url now newCache found newEntry aUrl |
	url := (aUrl := self asUrl) toText.
	now := Time totalSeconds.
	"Last time should be 'now' +- 3 seconds. +3 is needed if this image move
	to another time zone."
	newCache := self class entryCache
				select: [:each | now - 3 < each first
						and: [each first < (now + 3)]].
	found := newCache
				detect: [:each | each second = url]
				ifNone: [].
	found
		ifNotNil: [^ found third].
	newEntry := self privateEntriesFor: aUrl.
	newCache add: {now. url. newEntry}.
	self class entryCache: newCache.
	^ newEntry! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'cds 2/8/2005 15:26'!
fileExists: fileName
	"Does the file exist on this server directory?  fileName must be simple with no / or references to other directories."

	| |
	self isTypeFile ifTrue: [^ self fileNames includes: fileName].
	^ self entries anySatisfy: [:entry | entry name = fileName]! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/11/2005 16:01'!
findDirectoryEntries: anHtmlDocument 
	| preElement |
	preElement _ self findPreformattedItems: anHtmlDocument.
	preElement ifNil: [^#()].
	^self parsePreformattedRegion: preElement contents! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/17/2004 23:35'!
findPreformattedItems: anHtmlEntity
	| entity |
	anHtmlEntity tagName = 'pre' 
		ifTrue: [^anHtmlEntity].
	anHtmlEntity contents do: [ :each |
		entity _ self findPreformattedItems: each.
		entity ifNotNil: [^entity]].
	^nil! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'tak 4/17/2006 21:22'!
forceNewFileNamed: aString 
	^ self newFileNamed: aString! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/11/2005 15:56'!
fullPath: serverAndDirectory
	super fullPath: serverAndDirectory.
	directory last = $/ ifTrue: [directory := directory allButLast]! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'tak 7/12/2006 16:18'!
getBaselineCollection: c location: location forRevNumber: aNumber vcc: vcc

	| ret index endIndex |
	c openUrl: ('http://', self server, vcc) asUrl.
	ret _ c propFind: #('baseline-collection')
				depth: 0
				label: aNumber.
	index _ ret indexOfSubCollection: 'baseline-collection' startingAt: 1.
	index _ ret indexOfSubCollection: '<D:href>' startingAt: index + 1.
	endIndex _ ret indexOfSubCollection: '</D:href>' startingAt: index + 1.
	^ (ret copyFrom: index+8 to: endIndex -1).
! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'tak 7/12/2006 16:21'!
getBaselineRelativePath: c location: location

	| ret index endIndex |
	c openUrl: location asUrl.
	ret _ c propFind: #('baseline-relative-path xmlns="http://subversion.tigris.org/xmlns/dav/"')
				depth: 0
				label: nil.
	index _ ret indexOfSubCollection: 'baseline-relative-path>' startingAt: 1.
	endIndex _ ret indexOfSubCollection: '</' startingAt: index + 1.
	^ (ret copyFrom: index+23 to: endIndex -1).
! !

!DAVServerDirectory methodsFor: 'up/download' stamp: 'yo 11/17/2006 14:57'!
getFileNamed: fileNameOnServer

	^ (self readOnlyFileNamed: fileNameOnServer) contents.

! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'cds 2/18/2004 15:46'!
getOnly: nnn from: fileNameOnServer 
	| resp |
	resp _ HTTPSocket
				httpGet: (self fullNameFor: fileNameOnServer)
				accept: 'application/octet-stream'.
	^ resp next: nnn! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'tak 7/12/2006 16:21'!
getVersionControlledConfiguration: c location: location

	| ret index endIndex |
	c openUrl: location asUrl.
	ret := c propFind: #('version-controlled-configuration')
				depth: 0
				label: nil.
	index _ ret indexOfSubCollection: 'version-controlled-configuration' startingAt: 1.
	index _ ret indexOfSubCollection: '<D:href>' startingAt: index + 1.
	endIndex _ ret indexOfSubCollection: '</D:href>' startingAt: index + 1.
	^ (ret copyFrom: index+8 to: endIndex -1).
! !

!DAVServerDirectory methodsFor: 'testing' stamp: 'yo 7/27/2007 11:42'!
isProjectSwiki
	| path |
	path _ self server, self directory.
	(path endsWith: 'updates') ifTrue: [
		path _ path copyFrom: 1 to: (path size - 'updates' size).
	].
	(path endsWith: 'updates/') ifTrue: [
		path _ path copyFrom: 1 to: (path size - 'updates/' size).
	].
	
	^ (Utilities serverUrls includes: path) not! !

!DAVServerDirectory methodsFor: 'testing' stamp: 'cds 2/18/2004 00:21'!
isTypeHTTP
	^ true! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'cds 2/11/2005 15:55'!
localNameFor: fullName
	"Return the local part the given name."
	| tmp |
	tmp := fullName.
	fullName last = $/ ifTrue: [tmp := fullName allButLast].
	self
		splitName: tmp
		to: [:dirPath :localName | ^ localName]
! !

!DAVServerDirectory methodsFor: 'accessing' stamp: 'tak 4/18/2006 12:57'!
mimeTypesFor: fileName
	^ FileDirectory default mimeTypesFor: fileName! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'tak 7/10/2006 16:20'!
newestVersionForFileNamed: location

	| ret index endIndex |
	ret _ DAVClient
		openUrl: location asUrl
		while: [:c | c propFind: #('version-name')
				depth: 1
				label: nil].
	index _ ret indexOfSubCollection: 'version-name' startingAt: 1.
	index _ ret indexOfSubCollection: '>' startingAt: index + 1.
	endIndex _ ret indexOfSubCollection: '<' startingAt: index + 1.
	^ (ret copyFrom: index+1 to: endIndex -1) asNumber.
! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'tak 4/17/2006 22:54'!
newFileNamed: fullName 
	"Create a RemoteFileStream. If the file exists, and complain. fullName is
	directory path, and does include name of the server. Or it can just be a
	fileName. Only write the data upon close."
	| file remoteStrm |
	file := self asServerFileNamed: fullName.
	file readWrite.
	file isTypeFile
		ifTrue: [^ FileStream
				newFileNamed: (file fileNameRelativeTo: self)].
	remoteStrm := RemoteFileStream
				on: (String new: 2000).
	remoteStrm remoteFile: file.
	remoteStrm dataIsValid.
	"empty stream is the real contents!!"
	^ remoteStrm! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/18/2004 00:02'!
nextAnchorTagFrom: rs 
	| tag  |
	[tag _ rs next.
	tag tagName = 'a'
		or: [rs atEnd]] whileFalse.
	^rs atEnd
		ifTrue: [nil]
		ifFalse: [tag]! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 20:12'!
oldFileNamed: aName 
	| contents aUrl |
	aUrl :=  (self altUrl , '/' , aName) asUrl.
	self setUserAndPassIn: aUrl.
	contents := DAVClient
				openUrl: aUrl
				while: [:c | c get].
	^ (SwikiPseudoFileStream with: contents) reset; directory: self; localName: aName; yourself! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/18/2004 00:14'!
parseDate: aString 
	| toks date time |
	toks _ aString findTokens: ' '
					, (String with: Character cr)
					, (String with: Character lf).
	date _ toks first asDate.
	time _ toks second asTime.
	^ (TimeStamp date: date time: time) asSeconds! !

!DAVServerDirectory methodsFor: 'property access' stamp: 'tak 4/18/2006 14:47'!
parseDirectoryEntries: aStream 
	"Answer a collection of DirectoryEntry made from http response stream"
	| parser doc elements result |
	parser := XMLDOMParser on: aStream.
	parser useNamespaces: true.
	parser startDocument.
	parser parseDocument.
	doc := parser document.
	elements := (doc elementUnqualifiedAt: 'multistatus') elements.
	result := elements
				collect: [:each | self parsePropertyResponse: each].
	result := result
				select: [:each | each name isEmpty not].
	^ result! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/18/2004 15:44'!
parsePreformattedRegion: collection 
	| rs tag items name dateAndSizeString date size |
	rs _ collection readStream.
	5
		timesRepeat: [(tag _ self nextAnchorTagFrom: rs)
				ifNil: [^ nil]].
	"Anchor tag means file or directory name"
	(tag contents first printString beginsWith: '[Parent Directory]')
		ifFalse: [self error: 'DAV stream incorrectly formatted'].
	items _ OrderedCollection new.
	[tag _ self nextAnchorTagFrom: rs.
	tag
		ifNil: [^ items].
	name _ tag attributes at: 'href'.
	dateAndSizeString _ rs next text.
	date _ self parseDate: dateAndSizeString.
	size _ self parseSize: dateAndSizeString.
	items
		add: (DirectoryEntry
				name: name
				creationTime: date
				modificationTime: date
				isDirectory: (name endsWith: '/')
				fileSize: size).
	rs atEnd] whileFalse.
	^ items! !

!DAVServerDirectory methodsFor: 'property access' stamp: 'yo 9/16/2009 11:37'!
parsePropertyResponse: response 
	"Answer a DirectoryEntry made from response xml"
	| childName directoryString propStats aDictionary mtime value dirFlag fileSize contentString unescaped |
	directoryString := self directory.
	directoryString first = $/
		ifFalse: [directoryString := '/' , directoryString].
	directoryString last = $/
		ifFalse: [directoryString := directoryString , '/'].

	contentString := (response elementUnqualifiedAt: 'href') contentString.
	unescaped := directoryString unescapePercents.
	contentString size < unescaped size
		ifTrue: [childName := '']
		ifFalse: [childName := contentString allButFirst: unescaped size].
	(childName isEmpty not
			and: [childName last = $/])
		ifTrue: [childName := childName allButLast].
	childName.
	propStats := response elements
				select: [:e | e localName = 'propstat'].
	aDictionary := Dictionary new.
	propStats
		do: [:propStat | (propStat elementUnqualifiedAt: 'prop') ifNotNilDo: [:x | x  elements
				do: [:propElement | 
					value := propElement elements isEmpty
								ifTrue: [propElement contentString]
								ifFalse: [propElement elements first localName].
					aDictionary at: propElement localName put: value]]].
	mtime := fileSize := 0.
	dirFlag := false.
	mtime := aDictionary
				at: #getlastmodified
				ifPresent: [:v | [(TimeStamp
						fromString: (v allButFirst: 5)) asSeconds] on: Error do: [:ex | 0]].
	dirFlag := aDictionary
				at: #resourcetype
				ifPresent: [:v | v = #collection].
	fileSize := aDictionary
				at: #getcontentlength
				ifPresent: [:v | v asNumber].
	^ DirectoryEntry
		name: childName unescapePercents
		creationTime: 0
		modificationTime: mtime
		isDirectory: dirFlag
		fileSize: fileSize! !

!DAVServerDirectory methodsFor: 'private' stamp: 'cds 2/18/2004 15:43'!
parseSize: aString 
	| sizeString mult |
	sizeString _ (aString findTokens: ' '
					, (String with: Character cr)
					, (String with: Character lf)) last.
	mult _ 1.
	(sizeString endsWith: 'K')
		ifTrue: [mult _ 1024].
	(sizeString endsWith: 'M')
		ifTrue: [mult _ 1024 * 1024].
	^ (sizeString asNumber * mult) asInteger! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'tak 7/12/2006 16:36'!
pathForClient: connection revision: aNumber forFileNamed: fullName

	| location vcc br bc |
	location _ (self asUrl newFromRelativeText: fullName) toText.
	vcc _ self getVersionControlledConfiguration: connection location: location.
	br _ self getBaselineRelativePath: connection location: location.
	bc _ self getBaselineCollection: connection location: location forRevNumber: aNumber vcc: vcc.
	^ self typeForPrefs, '://', self server, bc, br.! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'yo 8/6/2009 20:12'!
pathForRevision: aNumber forFileNamed: fullName 
	| location aUrl |
	location := (self asUrl newFromRelativeText: fullName) toText.
	aUrl _ location asUrl.
	self setUserAndPassIn: aUrl.
	DAVClient
		openUrl: aUrl
		while: [:connection | ^ self
				pathForClient: connection
				revision: aNumber
				forFileNamed: fullName]! !

!DAVServerDirectory methodsFor: 'property access' stamp: 'yo 9/9/2009 01:23'!
privateEntries
	| location aUrl |
	location := 'http://' , self server , '/' , self directory , '/'.
	aUrl := location asUrl.
	self setUserAndPassIn: aUrl.
	^ self privateEntriesFor: aUrl.
! !

!DAVServerDirectory methodsFor: 'property access' stamp: 'yo 9/9/2009 01:39'!
privateEntriesFor: aUrl
	| ret |
	client := DAVClient openUrl: aUrl.
	ret := [client propFind: #('getlastmodified' 'getcontentlength' 'resourcetype' )
				depth: 1
				label: nil]
		on: Error do: [:ex | client close. (ex isKindOf: LoginFailedException) ifTrue: [ex outer]. ''].
	client multistatus ifTrue: [
		(client allLastResponseCodeFrom: ret) do: [:code |
			code first = $2 ifFalse: [(LoginFailedException protocolInstance: client) signal]]].
	^ ret ifNotEmpty: [self parseDirectoryEntries: ret readStream] ifEmpty: [#()]! !

!DAVServerDirectory methodsFor: 'file directory' stamp: 'yo 8/6/2009 20:12'!
putFile: fileStream named: fileNameOnServer 
	| fullURL dir |
	dir := self directory first = $/ ifTrue: [self directory allButFirst] ifFalse: [self directory].
	fullURL _ (self server , '/' , dir , '/' , fileNameOnServer encodeForHTTP) asUrl.
	self setUserAndPassIn: fullURL.
	fileStream binary.
	DAVClient openUrl: fullURL while: [ :c | c put: fileStream contents]
! !

!DAVServerDirectory methodsFor: 'ServerFile' stamp: 'tak 4/17/2006 22:28'!
realUrl
	^ self typeWithDefault asString , '://' , self pathName , '/'! !

!DAVServerDirectory methodsFor: 'svn protocol' stamp: 'tak 7/8/2006 16:12'!
revisionObjectsForFileNamed: fullName 
	| log |
	log := self revisionsForFileNamed: fullName.
	log size = 0
		ifTrue: [^ #()].
	^ self
		revisionObjectsForFileNamed: fullName
		fromXML: (XMLDOMParser parseDocumentFrom: log readStream)! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'yo 5/16/2006 17:32'!
revisionObjectsForFileNamed: n fromXML: anXMLElement

	| result r |
	result _ OrderedCollection new.
	anXMLElement elements first elements do: [:elem |
		r _ SVNRevision newFromXMLElement: elem forFileNamed: n.
		r ifNotNil: [result add: r].
	].
	^ result.
! !

!DAVServerDirectory methodsFor: 'svn protocol private' stamp: 'yo 8/6/2009 20:13'!
revisionsForFileNamed: fullName 
	| location newest aUrl |
	location _ (self asUrl newFromRelativeText: fullName) toText.
	newest _ self newestVersionForFileNamed: location.
	newest
		ifNil: [^ ''].
	aUrl := location asUrl.
	self setUserAndPassIn: aUrl.
	^ DAVClient
		openUrl: aUrl
		while: [:c | c
				logReportFrom: newest
				to: 1
				depth: 0]! !

!DAVServerDirectory methodsFor: 'svn protocol' stamp: 'tak 7/12/2006 16:39'!
revisionsFor: fullName do: aBlock 
	| revisions |
	revisions := self revisionObjectsForFileNamed: fullName.
	DAVClient
		openUrl: self asUrl
		while: [:connection | revisions
				do: [:each | 
					"NOTE: for some reason, 'asUrl toText' is needed. It
					converts !! to %21. I think it is not good idea, but is
					needed because URL is retrieved through URL object in
					#setInfoFromStream:. "
					each location: (self
							pathForClient: connection
							revision: each versionName
							forFileNamed: each name) asUrl toText.
					aBlock value: each]]! !

!DAVServerDirectory methodsFor: 'private' stamp: 'yo 8/6/2009 20:11'!
setUserAndPassIn: aUrl

	aUrl username: self user.
	aUrl password: self password.
! !

!DAVServerDirectory methodsFor: 'up/download' stamp: 'cds 2/18/2004 15:49'!
upLoadProject: projectName members: archiveMembers retry: aBool 
	self halt! !

!DAVServerDirectory methodsFor: 'up/download' stamp: 'cds 2/18/2004 16:00'!
upLoadProject: projectFile named: fileNameOnServer resourceUrl: resUrl retry: aBool 
	self
		putFile: projectFile
		named: fileNameOnServer
		retry: aBool.
	^true! !

DAVServerDirectory subclass: #DAVServerFile
	instanceVariableNames: 'fileName rwmode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAVServerDirectory'!

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:29'!
asStream
	"Return a RemoteFileStream (subclass of RWBinaryOrTextStream) on the contents of the remote file I represent.  For reading only.  This method is probably misnamed.  Maybe call it makeStream"

	^ self readOnlyFileNamed: self fileName! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:28'!
directoryUrl
	| ru |
	"A url to the directory this file is in"

	ru _ self realUrl.
	^ ru copyFrom: 1 to: (ru size - fileName size)! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:08'!
exists
	"Return true if the file exists on the server already"

	^ self fileExists: fileName! !

!DAVServerFile methodsFor: 'accessing' stamp: 'cds 2/8/2005 15:09'!
fileName
	^fileName! !

!DAVServerFile methodsFor: 'accessing' stamp: 'cds 2/8/2005 15:09'!
fileName: anObject
	fileName := anObject! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:23'!
fullPath: serverAndDirectory
	"Parse and save a full path.  Separate out fileName at the end."

	| delim ii |
	super fullPath: serverAndDirectory.		"set server and directory"
	self isTypeFile ifTrue: [
		fileName _  ''.
		^ self
	].
	delim _ self pathNameDelimiter.
	ii _ directory findLast: [:c | c = delim].
	ii = 0
		ifTrue: [self error: 'expecting directory and fileName']
		ifFalse: [fileName _ directory copyFrom: ii+1 to: directory size.
			directory _ (directory copyFrom: 1 to: directory size - fileName size - 1)].! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:28'!
localName

	^ self fileName! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:11'!
readOnly
	"Set the receiver to be read-only"

	rwmode _ false! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:09'!
readWrite
	"Set the receiver to be writable"

	rwmode _ true! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:28'!
realUrl
	"a fully expanded version of the url we represent.  Prefix the path with http: or ftp: or file:"

	self isTypeFile ifTrue: [
		self fileNameRelativeTo: self.
		^ urlObject toText
	].
	^ self typeWithDefault asString, '://', self pathName, '/', fileName	"note difference!!"
	! !

!DAVServerFile methodsFor: 'ServerFile' stamp: 'cds 2/8/2005 15:10'!
writable
	^ rwmode! !

LoginFailedException subclass: #HTTPLoginFailedException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAVServerDirectory'!

!HTTPLoginFailedException methodsFor: 'priv handling' stamp: 'tak 7/12/2006 21:29'!
defaultAction
	^ protocolInstance askNamePassword! !
